#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "config.h"

#define VERSION "1.4"

#define BUF_SIZE 256
#define MSG_SIZE 1024
#define MAX_LOG 2

#define DFCMD "df -k"
#define ERRPTCMD "errpt -a -s "
#define UPTIMECMD "uptime"

#if SYSTEM == BSD
#define PSCMD "ps -axo comm"
#else
#define PSCMD "ps -eo comm"
#endif

#if SYSTEM == SOLARIS
#define MEMCMD "swap -s"
#elif SYSTEM == TRU64
#define MEMCMD "/sbin/swapon -s"
#elif SYSTEM == AIX
#define MEMCMD "/usr/local/nagios/libexec/svmon -G"
#elif SYSTEM == BSD
#define MEMCMD "swapctl -s"
#else
#define MEMCMD "free"
#endif

#ifdef CHECK_DISK
struct disktab {
	char *name;
	int cmin;
	int wmin;
	struct disktab *next;
};

struct romtab {
	char *name;
	struct romtab *next;
};
#endif

#ifdef CHECK_PROC
struct proctab {
	char *name;
	int cmin;
	int wmin;
	int wmax;
	int cmax;
	int counter;
	struct proctab *next;
};
#endif

#if defined(CHECK_ERRPT) && SYSTEM == AIX
struct errpttab {
	char *label;
	char *state;
	struct errpttab *next;
};
#endif

void append_msg(char *, char *);


main(int argc, char *argv[])
{
#ifdef CHECK_DISK
	struct disktab *disktab_list;
	struct disktab *disktab_ptr;
	
	struct romtab *romtab_list;
	struct romtab *romtab_ptr;
#endif

#ifdef CHECK_PROC	
	struct proctab *proctab_list;
	struct proctab *proctab_ptr;
#endif

#ifdef CHECK_MEM
	int cswap;
	int wswap;
#endif

#ifdef CHECK_LOAD
	float cload05, cload10, cload15;
	float wload05, wload10, wload15;
	float load05, load10, load15;
#endif
	char buf[BUF_SIZE];
	char name[BUF_SIZE];
	char percent[BUF_SIZE];

	char cmsg[MSG_SIZE];
	char wmsg[MSG_SIZE];
	
	char *ptr;
	int rc, rc1, rc2, c;
	int cmin, wmin, wmax, cmax, total, used, free;
	
	FILE *fp;
#if SYSTEM == AIX
	struct errpttab *errpttab_list;
	struct errpttab *errpttab_ptr;

	char label[BUF_SIZE];
	char resource[BUF_SIZE];
	char descr[BUF_SIZE];
	char state[BUF_SIZE];
	char class;

	time_t t_time;
	struct tm *tm_time;
	char s_time[BUF_SIZE];
#endif
	*cmsg = '\0';
	*wmsg = '\0';
	
	if (argc == 2) {
		if (strcmp(argv[1], "-h") == 0) {
			if ((ptr = strchr(argv[0], '/')) == NULL)
				ptr = argv[0];
			else
				ptr++;

			printf("%s version " VERSION "\n", ptr);
			printf("\n%s performs the following checks:\n", ptr);
#ifdef CHECK_DISK
			printf("* disk space\n");
#endif
#ifdef CHECK_PROC
			printf("* process list\n");
#endif
#ifdef CHECK_MEM
			printf("* swap utilization\n");
#endif
#ifdef CHECK_LOAD
			printf("* load (run queue)\n");
#endif
#if defined(CHECK_ERRPT) && SYSTEM == AIX
			printf("* errpt\n");
#endif
#ifdef CHECK_SYSLOG
			printf("* syslog\n");
#endif
			printf("\nThis plugin takes no arguments but needs the following files:\n");
#ifdef CHECK_DISK
			printf("* " DISKTAB "\n");
			printf("* " ROMTAB "\n");
#endif
#ifdef CHECK_PROC
			printf("* " PROCTAB "\n");
#endif
#if defined(CHECK_ERRPT) && SYSTEM == AIX
			printf("* " ERRPTTAB "\n");
#endif
#ifdef CHECK_MEM
			printf("* " MEMRC "\n");
#if SYSTEM == AIX
			printf("* " MEMCMD "\n");
#endif
#endif
#ifdef CHECK_LOAD
			printf("* " LOADRC "\n");
#endif
#ifdef CHECK_SYSLOG
			printf("* " CSYSLOG "\n");
			printf("* " WSYSLOG "\n");
#endif
			exit(0);
		}
	}
	
	
	/* Read configuration file disktab */

#ifdef CHECK_DISK	
	fp = fopen(DISKTAB, "r");
	
	if (fp == NULL) {
		printf("Could not open " DISKTAB "\n");
		exit(2);
	}
	
	disktab_list = (struct disktab *) malloc(sizeof(struct disktab));
	disktab_list->next = NULL;
	disktab_ptr = disktab_list;
	
	while (fgets(buf, BUF_SIZE, fp) != NULL) {
	
		if ((ptr = strchr(buf, '#')) != NULL)
			*ptr = '\0';
	
		rc = sscanf(buf, "%s %d %d", name, &cmin, &wmin);
		
		if (rc != 3)
			continue;
		
		disktab_ptr->next = (struct disktab *) malloc(sizeof(struct disktab));
		disktab_ptr = disktab_ptr->next;
		disktab_ptr->next = NULL;
		
		disktab_ptr->name = (char *) malloc(sizeof(char) * (strlen(name) + 1));
		strcpy(disktab_ptr->name, name);
		
		disktab_ptr->cmin = cmin;
		disktab_ptr->wmin = wmin;
	}
	
	fclose(fp);
	
	
	/* Read configuration file romtab */
	
	fp = fopen(ROMTAB, "r");
	
	if (fp == NULL) {
		printf("Could not open " ROMTAB "\n");
		exit(2);
	}
	
	romtab_list = (struct romtab *) malloc(sizeof(struct romtab));
	romtab_list->next = NULL;
	romtab_ptr = romtab_list;
	
	while (fgets(buf, BUF_SIZE, fp) != NULL) {
	
		if ((ptr = strchr(buf, '#')) != NULL)
			*ptr = '\0';
	
		rc = sscanf(buf, "%s", name);
		
		if (rc != 1)
			continue;
		
		romtab_ptr->next = (struct romtab *) malloc(sizeof(struct romtab));
		romtab_ptr = romtab_ptr->next;
		romtab_ptr->next = NULL;
		
		romtab_ptr->name = (char *) malloc(sizeof(char) * (strlen(name) + 1));
		strcpy(romtab_ptr->name, name);
	}
	
	fclose(fp);
#endif


	/* Read configuration file proctab */

#ifdef CHECK_PROC	
	fp = fopen(PROCTAB, "r");
	
	if (fp == NULL) {
		printf("Could not open " PROCTAB "\n");
		exit(2);
	}
	
	proctab_list = (struct proctab *) malloc(sizeof(struct proctab));
	proctab_list->next = NULL;
	proctab_ptr = proctab_list;
	
	while (fgets(buf, BUF_SIZE, fp) != NULL) {
	
		if ((ptr = strchr(buf, '#')) != NULL)
			*ptr = '\0';
	
		rc = sscanf(buf, "%s %d %d %d %d", name, &cmin, &wmin, &wmax, &cmax);
		
		if (rc != 5)
			continue;
		
		proctab_ptr->next = (struct proctab *) malloc(sizeof(struct proctab));
		proctab_ptr = proctab_ptr->next;
		proctab_ptr->next = NULL;
		
		proctab_ptr->name = (char *) malloc(sizeof(char) * (strlen(name) + 1));
		strcpy(proctab_ptr->name, name);
		
		proctab_ptr->cmin = cmin;
		proctab_ptr->wmin = wmin;
		proctab_ptr->wmax = wmax;
		proctab_ptr->cmax = cmax;
		proctab_ptr->counter = 0;
	}
	
	fclose(fp);
#endif


	/* Read configuration file errpttab */

#if defined(CHECK_ERRPT) && SYSTEM == AIX
	fp = fopen(ERRPTTAB, "r");

	if (fp == NULL) {
		printf("Could not open " ERRPTTAB "\n");
		exit(2);
	}

	errpttab_list = (struct errpttab *) malloc(sizeof(struct errpttab));
	errpttab_list->next = NULL;
	errpttab_ptr = errpttab_list;

	while (fgets(buf, BUF_SIZE, fp) != NULL) {

		if ((ptr = strchr(buf, '#')) != NULL)
			*ptr = '\0';

		rc = sscanf(buf, "%s %s", label, state);

		if (rc != 2)
			continue;

		errpttab_ptr->next = (struct errpttab *) malloc(sizeof(struct errpttab));
		errpttab_ptr = errpttab_ptr->next;
		errpttab_ptr->next = NULL;

		errpttab_ptr->label = (char *) malloc(sizeof(char) * (strlen(label) + 1));
		strcpy(errpttab_ptr->label, label);

		errpttab_ptr->state = (char *) malloc(sizeof(char) * (strlen(state) + 1));
		strcpy(errpttab_ptr->state, state);
	}	

	fclose(fp);
#endif


	/* Read configuration file memrc */

#ifdef CHECK_MEM
	rc1 = rc2 = 0;
	
	fp = fopen(MEMRC, "r");
	
	if (fp == NULL) {
		printf("Could not open " MEMRC "\n");
		exit(2);
	}
	
	while (fgets(buf, BUF_SIZE, fp) != NULL) {
	
		if ((ptr = strchr(buf, '#')) != NULL)
			*ptr = '\0';
			
		if (strncmp(buf, "C_SWAP=", 7) == 0)
			rc1 = sscanf(buf + 7, "%d", &cswap);
		
		if (strncmp(buf, "W_SWAP=", 7) == 0)
			rc2 = sscanf(buf + 7, "%d", &wswap);
	}

	fclose(fp);

	if (rc1 != 1) {
		printf("Could not find parameter C_SWAP in " MEMRC "\n");
		exit(2);
	}
	
	if (rc2 != 1) {
		printf("Could not find parameter W_SWAP in " MEMRC "\n");
		exit(2);
	}
#endif	


	/* Read configuration file loadrc */
	
#ifdef CHECK_LOAD
	rc1 = rc2 = 0;

	fp = fopen(LOADRC, "r");
	
	if (fp == NULL) {
		printf("Could not open " LOADRC "\n");
		exit(2);
	}
	
	while (fgets(buf, BUF_SIZE, fp) != NULL) {
	
		if ((ptr = strchr(buf, '#')) != NULL)
			*ptr = '\0';
			
		if (strncmp(buf, "C_LOAD=", 7) == 0)
			rc1 = sscanf(buf + 7, "%f,%f,%f", &cload05, &cload10, &cload15);
		
		if (strncmp(buf, "W_LOAD=", 7) == 0)
			rc2 = sscanf(buf + 7, "%f,%f,%f", &wload05, &wload10, &wload15);
	}

	fclose(fp);
	
	if (rc1 == 0) {
		printf("Could not find parameter C_LOAD in " LOADRC "\n");
		exit(2);
	}
	
	if (rc2 == 0) {
		printf("Could not find parameter W_LOAD in " LOADRC "\n");
		exit(2);
	}
	
	if (rc1 != 3) {
		printf("Incorrect parameter C_LOAD in " LOADRC "\n");
		exit(2);
	}
	
	if (rc2 != 3) {
		printf("Incorrect parameter W_LOAD in " LOADRC "\n");
		exit(2);
	}
#endif


	/* Check system log */

#if defined(CHECK_ERRPT) && SYSTEM == AIX
	fp = fopen(ERRPT_TM, "r");
	
	if (fp == NULL) {
		fp = fopen(ERRPT_TM, "w");
		
		if (fp == NULL) {
			printf("Could not create " ERRPT_TM "\n");
			exit(2);
		}
		
		fclose(fp);
		strcpy(s_time, "0101000000");
	}
	
	else {
		if (fgets(s_time, BUF_SIZE, fp) == NULL) {
			printf("Could not read " ERRPT_TM "\n");
			exit(2);
		}
		
		fclose(fp);
	}
	
	time(&t_time);
	tm_time = localtime(&t_time);
	strftime(buf, BUF_SIZE, "%m%d%H%M%y", tm_time);
	
	fp = fopen(ERRPT_TM, "w");
		
	if (fp == NULL) {
		printf("Could not write to " ERRPT_TM "\n");
		exit(2);
	}
		
	fprintf(fp, "%s\n", buf);
	fclose(fp);
	
	strncpy(buf, ERRPTCMD, BUF_SIZE);
	strncat(buf, s_time, BUF_SIZE);
	
	fp = popen(buf , "r");
	
	if (fp == NULL) {
		printf("Could not start '%s'\n", buf);
		exit(2);
	}
	
	while (fgets(buf, BUF_SIZE, fp) != NULL) {
	
		if (strncmp(buf, "LABEL:", 6) == 0) {
			rc = sscanf(buf, "%*s %s", label);
			
			if (rc != 1)
				break;
		}

		if (strncmp(buf, "Class:", 6) == 0) {
			rc = sscanf(buf, "%*s %c", &class);

			if (rc != 1)
				break;
		}
		
		if (strncmp(buf, "Resource Name:", 14) == 0) {
			rc = sscanf(buf, "%*s %*s %s", resource);
			
			if (rc != 1)
				break;
		}
		
		if (strncmp(buf, "Description", 11) == 0) {
			fgets(buf, BUF_SIZE, fp);
			
			if (buf == NULL)
				break;
			
			if (strlen(buf) < BUF_SIZE && strlen(buf) > 0) {
				*(buf + strlen(buf) - 1) = '\0';
				strcpy(descr, buf);
			}

			else
				strcpy(descr, "");

			snprintf(buf, BUF_SIZE, "%s: %s (%s)", resource, descr, label);

			for (errpttab_ptr = errpttab_list->next; errpttab_ptr != NULL; errpttab_ptr = errpttab_ptr->next) {
				if (strcmp(errpttab_ptr->label, label) == 0) {

					if (strcmp(errpttab_ptr->state, "CRITICAL") == 0) {
						append_msg(cmsg, buf);
						break;
					}

					else if (strcmp(errpttab_ptr->state, "WARNING") == 0) {
						append_msg(wmsg, buf);
						break;
					}

					else
						break;
				}
			}

			if (errpttab_ptr == NULL) {
				if (class == 'H' || class == 'O' || class == 'U')
					append_msg(cmsg, buf);
				else
					append_msg(wmsg, buf);
			}
		}
	}
	
	fclose(fp);
#endif	

#ifdef CHECK_SYSLOG
	fp = fopen(CSYSLOG, "r");
	
	if (fp == NULL) {
		printf("Could not open " CSYSLOG "\n");
		exit(2);
	}
	
	for (c = 0; fgets(buf, BUF_SIZE, fp) != NULL && c < MAX_LOG; c++) {
		ptr = strchr(buf, '\n');
		
		if (ptr != NULL)
			*ptr = '\0';
			
		append_msg(cmsg, buf);
	}
	
	fclose(fp);
	
	fp = fopen(CSYSLOG, "w");
	
	if (fp == NULL) {
		printf("Could not write to " CSYSLOG "\n");
		exit(2);
	}
	
	fclose(fp);
	
	fp = fopen(WSYSLOG, "r");
	
	if (fp == NULL) {
		printf("Could not open " WSYSLOG "\n");
		exit(2);
	}
	
	for (c = 0; fgets(buf, BUF_SIZE, fp) != NULL && c < MAX_LOG; c++) {
		ptr = strchr(buf, '\n');
		
		if (ptr != NULL)
			*ptr = '\0';
			
		append_msg(wmsg, buf);
	}
	
	fclose(fp);
	
	fp = fopen(WSYSLOG, "w");
	
	if (fp == NULL) {
		printf("Could not write to " WSYSLOG "\n");
		exit(2);
	}
	
	fclose(fp);
#endif


	/* Check disk space */

#ifdef CHECK_DISK
	fp = popen(DFCMD, "r");
	
	if (fp == NULL) {
		printf("Could not start '" DFCMD "'\n");
		exit(2);
	}
	
	while (fgets(buf, BUF_SIZE, fp) != NULL) {
	
#if SYSTEM == AIX
		rc = sscanf(buf, "%*s %*s %d %s %*s %*s %s", &free, percent, name);
#else
		rc = sscanf(buf, "%*s %*s %*s %d %s %s", &free, percent, name);
#endif
		
		if (rc != 3)
			continue;
		
		if (strcmp(percent, "100%") == 0) {
		
			for (romtab_ptr = romtab_list->next; romtab_ptr != NULL; romtab_ptr = romtab_ptr->next) {
				if (strcmp(romtab_ptr->name, name) == 0)
					break;
				
			}
			
			if (romtab_ptr == NULL) {
#ifdef USE_SNPRINTF			
				snprintf(buf, BUF_SIZE, "Partition on mount point %s is full", name);
#else
				sprintf(buf, "Partition on mount point %s is full", name);
#endif
				append_msg(cmsg, buf);

				
				continue;
			}
		}
		
		for (disktab_ptr = disktab_list->next; disktab_ptr != NULL; disktab_ptr = disktab_ptr->next) {
		
			if (strcmp(disktab_ptr->name, name) != 0)
				continue;
		
			if (free < disktab_ptr->cmin) {
#ifdef USE_SNPRINTF
				snprintf(buf, BUF_SIZE, "Free space on mount point %s too low: %d (min: %d)", name, free, disktab_ptr->cmin);
#else
				sprintf(buf, "Free space on mount point %s too low: %d (min: %d)", name, free, disktab_ptr->cmin);
#endif
				append_msg(cmsg, buf);
			}
			
			else if (free < disktab_ptr->wmin) {
#ifdef USE_SNPRINTF
				snprintf(buf, BUF_SIZE, "Free space on mount point %s too low: %d (min: %d)", name, free, disktab_ptr->wmin);
#else
				sprintf(buf, "Free space on mount point %s too low: %d (min: %d)", name, free, disktab_ptr->wmin);
#endif
				append_msg(wmsg, buf);
			}
		}
	}
	
	fclose(fp);
#endif
	
	
	/* Check process list */
	
#ifdef CHECK_PROC
	fp = popen(PSCMD, "r");
	
	if (fp == NULL) {
		printf("Could not start '" PSCMD "'\n");
		exit(2);
	}
	
	while (fgets(buf, BUF_SIZE, fp) != NULL) {
		if ((ptr = strrchr(buf, '/')) == NULL)
			ptr = buf;
	
		for (proctab_ptr = proctab_list->next; proctab_ptr != NULL; proctab_ptr = proctab_ptr->next)		
			if (strstr(buf, proctab_ptr->name) != NULL)
				(proctab_ptr->counter)++;
	}
	
	pclose(fp);
	
	for (proctab_ptr = proctab_list->next; proctab_ptr != NULL; proctab_ptr = proctab_ptr->next) {
	
		if (proctab_ptr->counter < proctab_ptr->cmin) {
#ifdef USE_SNPRINTF
			snprintf(buf, BUF_SIZE, "Number of %s processes too low: %d (min: %d)", proctab_ptr->name, proctab_ptr->counter, proctab_ptr->cmin);
#else
			sprintf(buf, "Number of %s processes too low: %d (min: %d)", proctab_ptr->name, proctab_ptr->counter, proctab_ptr->cmin);
#endif
			append_msg(cmsg, buf);
		}
		
		else if (proctab_ptr->counter > proctab_ptr->cmax) {
#ifdef USE_SNPRINTF		
			snprintf(buf, BUF_SIZE, "Number of %s processes too high: %d (max: %d)", proctab_ptr->name, proctab_ptr->counter, proctab_ptr->cmax);
#else
			sprintf(buf, "Number of %s processes too high: %d (max: %d)", proctab_ptr->name, proctab_ptr->counter, proctab_ptr->cmax);
#endif
			append_msg(cmsg, buf);
		}
		
		else if (proctab_ptr->counter < proctab_ptr->wmin) {
#ifdef USE_SNPRINTF
			snprintf(buf, BUF_SIZE, "Number of %s processes too low: %d (min: %d)", proctab_ptr->name, proctab_ptr->counter, proctab_ptr->wmin);
#else
			sprintf(buf, "Number of %s processes too low: %d (min: %d)", proctab_ptr->name, proctab_ptr->counter, proctab_ptr->wmin);
#endif
			append_msg(wmsg, buf);
		}
		
		else if (proctab_ptr->counter > proctab_ptr->wmax) {
#ifdef USE_SNPRINTF
			snprintf(buf, BUF_SIZE, "Number of %s processes too high: %d (max: %d)", proctab_ptr->name, proctab_ptr->counter, proctab_ptr->wmax);
#else
			sprintf(buf, "Number of %s processes too high: %d (max: %d)", proctab_ptr->name, proctab_ptr->counter, proctab_ptr->wmax);
#endif
			append_msg(wmsg, buf);
		}
	}
#endif
	

	/* Check memory usage */

#ifdef CHECK_MEM
	fp = popen(MEMCMD, "r");

	if (fp == NULL) {
		printf("Could not start '" MEMCMD "'\n");
		exit(2);
	}

	free = 0;

	while (fgets(buf, BUF_SIZE, fp) != NULL) {
#if SYSTEM == SOLARIS || SYSTEM == BSD
		if (strncmp(buf, "total:", 6) == 0) {
			if ((ptr = strrchr(buf, 'k')) != NULL) {
				*ptr = '\0';
				
				if ((ptr = strrchr(buf, ' ')) != NULL)
					free = atoi(ptr + 1);
			}
		}
#elif SYSTEM == TRU64
		if (strstr(buf, "Available space:") != NULL) {
			rc = sscanf(buf, "%*s %*s %d %*s", &free);
			free = free / 8;
		}
#elif SYSTEM == AIX
		if (strncmp(buf, "pg space", 8) == 0) {
			rc = sscanf(buf, "%*s %*s %d %d", &total, &used);
			free = (total - used) * 4;
		}
#else
		if (strncmp(buf, "Swap:", 5) == 0)
			rc = sscanf(buf, "%*s %*s %*s %d", &free);
#endif
	}

	pclose(fp);

	if (free < cswap) {
#ifdef USE_SNPRINTF
		snprintf(buf, BUF_SIZE, "Free swap space too low: %d (min: %d)", free, cswap);
#else
		sprintf(buf, "Free swap space too low: %d (min: %d)", free, cswap);
#endif
		append_msg(cmsg, buf);
	}
	
	else if (free < wswap) {
#ifdef USE_SNPRINTF
		snprintf(buf, BUF_SIZE, "Free swap space too low: %d (min: %d)", free, wswap);
#else
		sprintf(buf, "Free swap space too low: %d (min: %d)", free, wswap);
#endif
		append_msg(wmsg, buf);
	}
#endif

	
	/* Check system load */

#ifdef CHECK_LOAD
	fp = popen(UPTIMECMD, "r");
	
	if (fp == NULL) {
		printf("Could not start '" UPTIMECMD "'\n");
		exit(2);
	}
	
	fgets(buf, BUF_SIZE, fp);
	rc = sscanf(strrchr(buf, ':') + 1, "%f,%f,%f", &load05, &load10, &load15);

	pclose(fp);	
	
	if (load05 > cload05) {
#ifdef USE_SNPRINTF
		snprintf(buf, BUF_SIZE, "System load (5 min avg) too high: %.2f (max: %.2f)", load05, cload05);
#else
		sprintf(buf, "System load (5 min avg) too high: %.2f (max: %.2f)", load05, cload05);
#endif
		append_msg(cmsg, buf);
	}

	else if (load10 > cload10) {
#ifdef USE_SNPRINTF
		snprintf(buf, BUF_SIZE, "System load (10 min avg) too high: %.2f (max: %.2f)", load10, cload10);
#else
		sprintf(buf, "System load (10 min avg) too high: %.2f (max: %.2f)", load10, cload10);
#endif
		append_msg(cmsg, buf);
	}

	else if (load15 > cload15) {
#ifdef USE_SNPRINTF
		snprintf(buf, BUF_SIZE, "System load (15 min avg) too high: %.2f (max: %.2f)", load15, cload15);
#else
		sprintf(buf, "System load (15 min avg) too high: %.2f (max: %.2f)", load15, cload15);
#endif
		append_msg(cmsg, buf);
	}

	else if (load05 > wload05) {
#ifdef USE_SNPRINTF
		snprintf(buf, BUF_SIZE, "System load (5 min avg) too high: %.2f (max: %.2f)", load05, wload05);
#else
		sprintf(buf, "System load (5 min avg) too high: %.2f (max: %.2f)", load05, wload05);
#endif
		append_msg(wmsg, buf);
	}

	else if (load10 > wload10) {
#ifdef USE_SNPRINTF
		snprintf(buf, BUF_SIZE, "System load (10 min avg) too high: %.2f (max: %.2f)", load10, wload10);
#else
		sprintf(buf, "System load (10 min avg) too high: %.2f (max: %.2f)", load10, wload10);
#endif
		append_msg(wmsg, buf);
	}

	else if (load15 > wload15) {
#ifdef USE_SNPRINTF
		snprintf(buf, BUF_SIZE, "System load (15 min avg) too high: %.2f (max: %.2f)", load15, wload15);
#else
		sprintf(buf, "System load (15 min avg) too high: %.2f (max: %.2f)", load15, wload15);
#endif
		append_msg(wmsg, buf);
	}	
#endif


	/* Print summary on stdout and calculate exit code */
	
	if (*cmsg != '\0') {
		printf("CRITICAL: %s", cmsg);
	}
	
	if (*wmsg != '\0') {
		if (*cmsg != '\0')
			printf(", ");
		
		printf("WARNING: %s", wmsg);
	}
	
	if (*cmsg == '\0' && *wmsg == '\0')
		printf("Status is OK");
	
	putchar('\n');
	
	if (*cmsg != '\0')
		exit(2);
		
	if (*wmsg != '\0')
		exit(1);
		
	exit(0);
}


void append_msg(char *msg, char *str)
{
	if (*msg == '\0') {
		if (strlen(str) < MSG_SIZE)
			strcpy(msg, str);
	}
	
	else {
		if (strlen(msg) + strlen(str) < MSG_SIZE - 3) {
			strcat(msg, ", ");
			strcat(msg, str);
		}
	}
}
