#!/bin/sh

BASEDIR=`dirname $0`
USER0="root"
GROUP0=`awk 'BEGIN { FS = ":" } $3 == 0 { print $1 }' < /etc/group`
WHOAMI=`whoami`

if [ "$WHOAMI" != "root" ]
then
	echo "Error: Please rerun this script as root"
	exit 1
fi

grep "^nagios:" /etc/group > /dev/null

if [ $? -ne 0 ]
then
	echo "Error: Please create group nagios and rerun this script"
	exit 1
fi

grep "^nagios:" /etc/passwd > /dev/null

if [ $? -ne 0 ]
then
	echo "Error: Please create user nagios and rerun this script"
	exit 1
fi

if [ ! -d "/usr/local" ]
then
	echo "Creating directory /usr/local"
	mkdir /usr/local
	chown $USER0 /usr/local
	chgrp $GROUP0 /usr/local
	chmod 755 /usr/local
fi

if [ ! -d "/usr/local/nagios" ]
then
	echo "Creating directory /usr/local/nagios"
	mkdir /usr/local/nagios
	chown $USER0 /usr/local/nagios
	chgrp $GROUP0 /usr/local/nagios
	chmod 755 /usr/local/nagios
fi

if [ ! -d "/usr/local/nagios/etc" ]
then
	echo "Creating directory /usr/local/nagios/etc"
	mkdir /usr/local/nagios/etc
	chown $USER0 /usr/local/nagios/etc
	chgrp $GROUP0 /usr/local/nagios/etc
	chmod 755 /usr/local/nagios/etc
fi

if [ ! -d "/usr/local/nagios/libexec" ]
then
	echo "Creating directory /usr/local/nagios/libexec"
	mkdir /usr/local/nagios/libexec
	chown $USER0 /usr/local/nagios/libexec
	chgrp $GROUP0 /usr/local/nagios/libexec
	chmod 755 /usr/local/nagios/libexec
fi

if [ ! -d "/var/local" ]
then
	echo "Creating directory /var/local"
	mkdir /var/local
	chown $USER0 /var/local
	chgrp $GROUP0 /var/local
	chmod 755 /var/local
fi

if [ ! -d "/var/local/nagios" ]
then
	echo "Creating directory /var/local/nagios"
	mkdir /var/local/nagios
	chown $USER0 /var/local/nagios
	chgrp $GROUP0 /var/local/nagios
	chmod 755 /var/local/nagios
fi

if [ ! -f "/usr/local/nagios/etc/disktab" ]
then
	echo "Copying example file disktab to /usr/local/nagios/etc"
	cp -p "$BASEDIR/etc/disktab" /usr/local/nagios/etc
	chown $USER0 /usr/local/nagios/etc/disktab
	chgrp $GROUP0 /usr/local/nagios/etc/disktab
	chmod 644 /usr/local/nagios/etc/disktab
fi

if [ ! -f "/usr/local/nagios/etc/romtab" ]
then
	echo "Copying example file romtab to /usr/local/nagios/etc"
	cp -p "$BASEDIR/etc/romtab" /usr/local/nagios/etc
	chown $USER0 /usr/local/nagios/etc/romtab
	chgrp $GROUP0 /usr/local/nagios/etc/romtab
	chmod 644 /usr/local/nagios/etc/romtab
fi

if [ ! -f "/usr/local/nagios/etc/proctab" ]
then
	echo "Copying example file proctab to /usr/local/nagios/etc"
	cp -p "$BASEDIR/etc/proctab" /usr/local/nagios/etc
	chown $USER0 /usr/local/nagios/etc/proctab
	chgrp $GROUP0 /usr/local/nagios/etc/proctab
	chmod 644 /usr/local/nagios/etc/proctab
fi

if [ ! -f "/usr/local/nagios/etc/errpttab" ]
then
	echo "Copying example file errpttab to /usr/local/nagios/etc"
	cp -p "$BASEDIR/etc/errpttab" /usr/local/nagios/etc
	chown $USER0 /usr/local/nagios/etc/errpttab
	chgrp $GROUP0 /usr/local/nagios/etc/errpttab
	chmod 644 /usr/local/nagios/etc/errpttab
fi

if [ ! -f "/usr/local/nagios/etc/memrc" ]
then
	echo "Copying example file memrc to /usr/local/nagios/etc"
	cp -p "$BASEDIR/etc/memrc" /usr/local/nagios/etc
	chown root /usr/local/nagios/etc/memrc
	chgrp $GROUP0 /usr/local/nagios/etc/memrc
	chmod 644 /usr/local/nagios/etc/memrc
fi

if [ ! -f "/usr/local/nagios/etc/loadrc" ]
then
	echo "Copying example file loadrc to /usr/local/nagios/etc"
	cp -p "$BASEDIR/etc/loadrc" /usr/local/nagios/etc
	chown $USER0 /usr/local/nagios/etc/loadrc
	chgrp $GROUP0 /usr/local/nagios/etc/loadrc
	chmod 644 /usr/local/nagios/etc/loadrc
fi

if [ -x "$BASEDIR/check_sys" ]
then
	echo "Copying executable file check_sys to /usr/local/nagios/libexec"
	cp -p "$BASEDIR/check_sys" /usr/local/nagios/libexec
	chown $USER0 /usr/local/nagios/libexec/check_sys
	chgrp $GROUP0 /usr/local/nagios/libexec/check_sys
	chmod 755 /usr/local/nagios/libexec/check_sys
else
	echo "Error: Did not find executable file check_sys in directory $BASEDIR"
	echo
	echo "Please compile file check_sys.c with the following command:"
	echo "cc -o check_sys check_sys.c"
	echo
	echo "After compiling please rerun this script"
	exit 1
fi

if [ ! -f "/var/local/nagios/critical.log" ]
then
	echo "Creating file /var/local/nagios/critical.log"
	touch /var/local/nagios/critical.log
	chown nagios /var/local/nagios/critical.log
	chgrp nagios /var/local/nagios/critical.log
	chmod 644 /var/local/nagios/critical.log
fi

if [ ! -f "/var/local/nagios/warning.log" ]
then
	echo "Creating file /var/local/nagios/warning.log"
	touch /var/local/nagios/warning.log
	chown nagios /var/local/nagios/warning.log
	chgrp nagios /var/local/nagios/warning.log
	chmod 644 /var/local/nagios/warning.log
fi

echo "Installation is finished!"
